/* -*- mode: c++; tab-width: 4; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef QT4
#include <QGroupBox>
#include <QHBoxLayout>
#include <QButtonGroup>
#include <QLabel>
#include <QHashIterator>
#else
#include <qvgroupbox.h>
#include <qhgroupbox.h>
#include <qvbox.h>
#include <qhbuttongroup.h>
#include <qlabel.h>
#include <qmap.h>
#endif

#include <qlayout.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qtextcodec.h>
#include <qfileinfo.h>
#include "simplefiledialog.h"
#include "mplayerconfig.h"
#include "mplayerbackendaddon.h"

#include "compathack.h"

/* MPlayerConfiguration */

MPlayerConfiguration::MPlayerConfiguration()
    :	mplayerPath("mplayer"),
        isOverlayEnabled(true),
        isFullScreenEnabled(true),
        isDoubleBufferingEnabled(true),
        framedropMethod(FRAMEDROP_NORMAL),
        isCacheEnabled(true),
        cacheSize(256),
        isPrefetchFileEnabled(true),
        isAdditionalOptionsEnabled(false),
        additionalOptions(""),
        machineCategory_(MACHINE_GENERIC),
        machineName_("")
{
    detectMachine();
    // GENERIC機種ではデフォルトでオーバレイを有効にしない
    if (machineCategory() == MACHINE_GENERIC) {
        isOverlayEnabled = false;
        isDoubleBufferingEnabled = false;
    }
}

void MPlayerConfiguration::detectMachine()
{
#ifdef QTOPIA
    // まず、bvddが動いているかを確認し、動いていればSL-C3000系であると判断する
    // bvddが動いていなければ、SL-C3000系であってもGENERIC機種とする
    QFile modulesFile("/proc/modules");
    if (modulesFile.open(IO_ReadOnly)) {
        QTextStream s(&modulesFile);
        QString line;
        while ((line = s.readLine()) != QString::null) {
            if (line.startsWith("bvdd "))
                machineCategory_ = MACHINE_SLC3000;
        }
    }

    QFile productFile("/proc/deviceinfo/product");
    if (productFile.open(IO_ReadOnly)) {
        QTextStream s(&productFile);
        s >> machineName_;
        if (machineName_ == "SL-C860" || machineName_ == "SL-C760" ||
            machineName_ == "SL-C750" || machineName_ == "SL-C700")
            machineCategory_ = MACHINE_SLC700;
    } else
        machineName_ = "Unknown";
#else
    machineName_ = "Unknown";
#endif
}

void MPlayerConfiguration::write(Config &cf) const
{
    cf.setGroup("MPlayer");
    cf.writeEntry("mplayerPath", qGlobalConfig.getRelativeFilePath(mplayerPath));
    cf.writeEntry("isFullScreenEnabled", isFullScreenEnabled);
    cf.writeEntry("isDoubleBufferingEnabled", isDoubleBufferingEnabled);
    cf.writeEntry("framedropMethod", framedropMethod);
    cf.writeEntry("isCacheEnabled", isCacheEnabled);
    cf.writeEntry("cacheSize", cacheSize);
    cf.writeEntry("isPrefetchFileEnabled", isPrefetchFileEnabled);
    cf.writeEntry("isAdditionalOptionsEnabled", isAdditionalOptionsEnabled);
    cf.writeEntry("additionalOptions", additionalOptions);

    cf.setGroup("MPlayerArguments");
    cf.clearGroup();

#ifdef QT4
    QHashIterator<QString, QString> it(formatMPlayerArguments);
    while (it.hasNext())
    {
        it.next();
        cf.writeEntry(it.key(), it.value());
    }
#else
    QMapConstIterator<QString, QString> it;
    for (it = formatMPlayerArguments.begin(); it != formatMPlayerArguments.end(); ++it)
    {
        cf.writeEntry(it.key(), it.data());
    }
#endif
}

void MPlayerConfiguration::read(Config &cf)
{
    cf.setGroup("MPlayer");
    mplayerPath = qGlobalConfig.resolveRelativeFilePath(cf.readEntry("mplayerPath", mplayerPath));
    isFullScreenEnabled = cf.readBoolEntry("isFullScreenEnabled", isFullScreenEnabled);
    isDoubleBufferingEnabled = cf.readBoolEntry("isDoubleBufferingEnabled", isDoubleBufferingEnabled);
    framedropMethod = static_cast<FramedropMethod>(cf.readNumEntry("framedropMethod", framedropMethod));
    isCacheEnabled = cf.readBoolEntry("isCacheEnabled", isCacheEnabled);
    cacheSize = cf.readNumEntry("cacheSize", cacheSize);
    isPrefetchFileEnabled = cf.readBoolEntry("isPrefetchFileEnabled", isPrefetchFileEnabled);
    isAdditionalOptionsEnabled = cf.readBoolEntry("isAdditionalOptionsEnabled", isAdditionalOptionsEnabled);
    additionalOptions = cf.readEntry("additionalOptions", additionalOptions);

    cf.setGroup("MPlayerArguments");
    formatMPlayerArguments.clear();

    for (int i = 0; i < qConfig.formatFileExtensions.count(); ++i)
    {
        QString formats = qConfig.formatFileExtensions[i];
        QString args = cf.readEntry(formats);

        if (!args.isNull())
            formatMPlayerArguments.insert(formats, args);
    }
}

QString MPlayerConfiguration::mplayerBinLocation()
{
#ifdef OSX
    QString t(qGlobalConfig.appPath() + "../Resources/" + mplayerPath);
    if (QFile::exists(t))
        return t;
#else
    QString t(qGlobalConfig.appPath() + mplayerPath);

#ifdef WINDOWS
#ifdef QT4
    if (!t.endsWith(".exe", Qt::CaseInsensitive))
        t += ".exe";
#else
    if (!t.endsWith(".exe", false))
        t += ".exe";
#endif
#endif

    if (QFile::exists(t))
        return t;
#endif

    return mplayerPath;
}

const QString MPlayerConfiguration::mplayerArguments(QString fileExtension)
{
#ifdef QT4
    fileExtension = fileExtension.toLower();

    for (int i = 0; i < qConfig.formatFileExtensions.count(); ++i)
    {
        QString fileExtensions = qConfig.formatFileExtensions[i];
        if (fileExtensions.contains(fileExtension, Qt::CaseInsensitive))
            return formatMPlayerArguments[fileExtensions];
    }
#else
    fileExtension = fileExtension.lower();

    for (int i = 0; i < qConfig.formatFileExtensions.count(); ++i)
    {
        QString fileExtensions = qConfig.formatFileExtensions[i];
        if (qConfig.formatFileExtensions[i].contains(fileExtension, false))
            return formatMPlayerArguments[fileExtensions];
    }
#endif

    return QString::null;
}

void MPlayerConfiguration::setFormatSpecificConfigDefaults(Configuration &config)
{
#ifdef LOWPERF
    formatMPlayerArguments.insert("ogg,oga", "-ac vorbis");
    formatMPlayerArguments.insert("mp3", "-ac mad -hr-mp3-seek");
#endif
}

/* MPlayerConfig */

MPlayerConfig::MPlayerConfig(Configuration& configuration, QWidget *parent)
    : ConfigView(configuration, parent)
{
}

MPlayerConfig::~MPlayerConfig()
{
}

/*! \a config を元にして各Widgetの状態を設定します。
 */
void MPlayerConfig::setStatesByConfiguration(const Configuration& config)
{
    ConfigView::setStatesByConfiguration(config);
    MPlayerConfiguration &mplayerConfig = gMPlayerBackendAddOn.config();

    if (mplayerConfig.machineCategory() == MPlayerConfiguration::MACHINE_GENERIC) {
        isOverlayEnabled_->setChecked(false);
        isOverlayEnabled_->hide();
    } else {
        isOverlayEnabled_->setChecked(mplayerConfig.isOverlayEnabled);
        isOverlayEnabled_->show();
    }
    isDoubleBufferingEnabled_->setChecked(mplayerConfig.isDoubleBufferingEnabled);
#ifdef QT4
    reinterpret_cast<QRadioButton *>(framedropMethod_->button(mplayerConfig.framedropMethod))->setChecked(true);
#else
    reinterpret_cast<QRadioButton *>(framedropMethod_->find(mplayerConfig.framedropMethod))->setChecked(true);
#endif
    isAdditionalOptionsEnabled_->setChecked(mplayerConfig.isAdditionalOptionsEnabled);

    isCacheEnabled_->setChecked(mplayerConfig.isCacheEnabled);
    cacheSize_->setValue(mplayerConfig.cacheSize);
    isPrefetchFileEnabled_->setChecked(mplayerConfig.isPrefetchFileEnabled);

    additionalOptions_->setText(mplayerConfig.additionalOptions);
    additionalOptions_->setEnabled(mplayerConfig.isAdditionalOptionsEnabled);
    mplayerPath_->setText(mplayerConfig.mplayerPath);
}

void MPlayerConfig::saveStatesToConfiguration()
{
    MPlayerConfiguration &config = gMPlayerBackendAddOn.config();

    config.isOverlayEnabled = isOverlayEnabled_->isChecked();
    config.isDoubleBufferingEnabled = isDoubleBufferingEnabled_->isChecked();

#ifdef QT4
    config.framedropMethod = static_cast<MPlayerConfiguration::FramedropMethod>(framedropMethod_->checkedId());
#else
    config.framedropMethod = static_cast<MPlayerConfiguration::FramedropMethod>(framedropMethod_->id(framedropMethod_->selected()));
#endif

    config.isCacheEnabled = isCacheEnabled_->isChecked();
    config.cacheSize = cacheSize_->value();
    config.isPrefetchFileEnabled = isPrefetchFileEnabled_->isChecked();
    config.isAdditionalOptionsEnabled = isAdditionalOptionsEnabled_->isChecked();
    config.additionalOptions = additionalOptions_->text();
    config.mplayerPath = mplayerPath_->text();
}

/*! 使用するWidgetを確保する
*/
void MPlayerConfig::createWidgets()
{
    ConfigView::createWidgets();

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->setSpacing(4);

#ifdef QT4
    QGroupBox *group = new QGroupBox(tr("MPlayer settings"), widget());
    QVBoxLayout *groupLayout = new QVBoxLayout(group);
#else
    QVGroupBox *group = new QVGroupBox(tr("MPlayer settings"), widget());
#endif

    layout->addWidget(group);

    isOverlayEnabled_ = new QCheckBox(tr("Enable video overlay (BVDD / ATI W100)"), group);
    isDoubleBufferingEnabled_ = new QCheckBox(tr("Enable double buffering (for OSD and smoother playback)"), group);

    // -------------------------------------------------------------

    QWidget *hbox = new QWidget(group);
    QHBoxLayout *hboxlayout = new QHBoxLayout(hbox);
    hboxlayout->setMargin(0);
    hboxlayout->setSpacing(4);

    isCacheEnabled_ = new QCheckBox(tr("Enable file cache:"), hbox);
    hboxlayout->addWidget(isCacheEnabled_);

#ifdef QT4
    cacheSize_ = new QSpinBox(hbox);
    cacheSize_->setMinimum(128);
    cacheSize_->setMaximum(16384);
    cacheSize_->setSingleStep(128);
#else
    cacheSize_ = new QSpinBox(128, 16384, 128, hbox);
#endif
    hboxlayout->addWidget(cacheSize_);
    cacheSize_->setSuffix(" KiB");

    // -------------------------------------------------------------

    isPrefetchFileEnabled_ = new QCheckBox(tr("Enable file prefetching"), group);

#ifdef QT4
    QGroupBox *framedropMethodGroupBox = new QGroupBox(tr("Framedrop method"), group);
    QRadioButton *framedropMethodNone = new QRadioButton(tr("None"), framedropMethodGroupBox);
    QRadioButton *framedropMethodNormal = new QRadioButton(tr("Normal"), framedropMethodGroupBox);
    QRadioButton *framedropMethodHard = new QRadioButton(tr("Hard (may crash MPlayer)"), framedropMethodGroupBox);

    QHBoxLayout *framedropMethodLayout = new QHBoxLayout(framedropMethodGroupBox);
    framedropMethodLayout->setSpacing(0);
    framedropMethodLayout->addWidget(framedropMethodNone);
    framedropMethodLayout->addWidget(framedropMethodNormal);
    framedropMethodLayout->addWidget(framedropMethodHard);

    framedropMethod_ = new QButtonGroup(framedropMethodGroupBox);
    framedropMethod_->addButton(framedropMethodNone, MPlayerConfiguration::FRAMEDROP_NONE);
    framedropMethod_->addButton(framedropMethodNormal, MPlayerConfiguration::FRAMEDROP_NORMAL);
    framedropMethod_->addButton(framedropMethodHard, MPlayerConfiguration::FRAMEDROP_HARD);
    framedropMethod_->setExclusive(true);
#else
    framedropMethod_ = new QHButtonGroup(tr("Framedrop method"), group);
    framedropMethod_->setExclusive(true);
    QRadioButton *framedropMethodNone = new QRadioButton(tr("None"), framedropMethod_);
    QRadioButton *framedropMethodNormal = new QRadioButton(tr("Normal"), framedropMethod_);
    QRadioButton *framedropMethodHard = new QRadioButton(tr("Hard (may crash MPlayer)"), framedropMethod_);
#endif

    isAdditionalOptionsEnabled_ = new QCheckBox(tr("Use additional options:"), group);
    additionalOptions_ = new QLineEdit(group);

#ifdef QT4
    groupLayout->addWidget(isOverlayEnabled_);
    groupLayout->addWidget(isDoubleBufferingEnabled_);
    groupLayout->addWidget(hbox);
    groupLayout->addWidget(isPrefetchFileEnabled_);
    groupLayout->addWidget(framedropMethodGroupBox);
    groupLayout->addWidget(isAdditionalOptionsEnabled_);
    groupLayout->addWidget(additionalOptions_);
#endif

    // ----------------

#ifdef QT4
    QGroupBox *pathGroup = new QGroupBox(tr("MPlayer location"), widget());
    QHBoxLayout *pathGroupLayout = new QHBoxLayout(pathGroup);
#else
    QHGroupBox *pathGroup = new QHGroupBox(tr("MPlayer location"), widget());
#endif

    layout->addWidget(pathGroup);

    mplayerPath_ = new QLineEdit(pathGroup);
    QPushButton *openButton = new QPushButton(tr("..."), pathGroup);
    connect(openButton, SIGNAL(clicked()), this, SLOT(execMPlayerPathDialog()));

#ifdef QT4
    pathGroupLayout->addWidget(mplayerPath_);
    pathGroupLayout->addWidget(openButton);
#endif
}

void MPlayerConfig::setDefaultSettings()
{

}

void MPlayerConfig::execMPlayerPathDialog()
{
    QString selectedFilename = SimpleFileDialog::getOpenFileName(
        QFileInfo(mplayerPath_->text()).filePath(),
        "All Files (*.*)",
        this,
        "locate mplayer dialog",
        tr("Locate MPlayer binary")
    );

    if (!selectedFilename.isNull())
        mplayerPath_->setText(selectedFilename);
}

/* MPlayerFormatsConfig */

MPlayerFormatsConfig::MPlayerFormatsConfig(QWidget *parent)
    :	PlaybackBackendConfigWidget(parent)
{
    new QHBoxLayout(this);

    QLabel *mplayerArgumentsLabel = new QLabel(tr("MPlayer Arguments:"), this);
    mplayerArgumentsLineEdit_ = new QLineEdit(this);
    mplayerArgumentsLineEdit_->setFixedWidth(250);

#ifdef QT4
    this->layout()->addWidget(mplayerArgumentsLabel);
    this->layout()->addWidget(mplayerArgumentsLineEdit_);
#else
    this->layout()->add(mplayerArgumentsLabel);
    this->layout()->add(mplayerArgumentsLineEdit_);
#endif
}

MPlayerFormatsConfig::~MPlayerFormatsConfig()
{

}

void MPlayerFormatsConfig::setStatesByFormats(const QString& formats)
{
    orgFormats_ = formats;
    formats_ = formats;

    QString arguments = gMPlayerBackendAddOn.config().formatMPlayerArguments[formats];
    mplayerArgumentsLineEdit_->setText(arguments);
}

void MPlayerFormatsConfig::formatsChanged(const QString& formats)
{
    formats_ = formats;
}

void MPlayerFormatsConfig::saveStates()
{
   gMPlayerBackendAddOn.config().formatMPlayerArguments.remove(orgFormats_);
   gMPlayerBackendAddOn.config().formatMPlayerArguments.insert(formats_, mplayerArgumentsLineEdit_->text());
}
